/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//=====================================================================================================================
/// @brief		Utility assertion macros and functions. Brings up a windows dialog box on error and shows what happened 
///				and where it happened at.
//=====================================================================================================================

#include "Precompiled.h"
#include "Game\GameCore.h"
#include "Windows\Window.h"

//=====================================================================================================================
/// @brief		This is the actual assertion fail function. Do not call this directly.
/// @param		condition - whether the condition is true or false
/// @param		message - message to display
/// @param		line - line number of the assertion
/// @param		function - function of the assertion
/// @param		file - file of the assertion
//=====================================================================================================================

#ifdef DEBUG

void Assertion_Fail
( 
	const char *	condition	,
	const char *	message		,
	unsigned		line		,
	const char *	function	,
	const char *	file
)
{
	// Buffer to print the message to:
	
	char buffer[2048];

	// Print the assertion message:
	
	sprintf_s
	(
		buffer						,
		2047						,
		"Condition failed: %s \n"
		"Line: %u \n"
		"Function: %s \n"
		"File: %s \n"
		"\n\n"
		"%s",
		condition,
		line,
		function,
		file,
		message
	);

	// If the game has a window then hide it:

	if ( Game::GameCore::GetSingleton() )
	{
		if ( Game::GameCore::GetSingleton()->GetWindow() )
		{
			Game::GameCore::GetSingleton()->GetWindow()->SetVisible(false);
		}
	}
	
	// Show a message box:
	
	MessageBoxA
	(
		0	,
		buffer	,
		"A debug assertion has failed"	,
		MB_OK | MB_ICONEXCLAMATION | MB_SYSTEMMODAL
	);

	// Trigger a breakpoint
	
	DebugBreak();
	
	// Exit
	
	exit(1);
}

#endif
