/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef RANDOM_H
#define RANDOM_H

//=====================================================================================================================
//=====================================================================================================================

#include "Base\Types.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	/// @brief		This class represents a random number generator and allows numbers to be generated randomly.
	//=================================================================================================================

	class Random
	{
		public:
		
			//=========================================================================================================
			// Functions
			//=========================================================================================================
						
				/// Default constructor: creates the random number generator and seeds based on current time
				
				Random();
				
				/// Overloaded constructor: creates the number generator and sets a seed manually
				
				Random( unsigned seed );
				
				/// Sets the random number generator seed:
				
				inline void SetSeed( unsigned seed ){ m_seed = seed; }
				
				/// Returns the seed of the random number generator
				
				inline unsigned GetSeed(){ return m_seed; }
				
				/// Gives a random float between 0 and 1

				float NextFloat();

				/// Gives a random float between two numbers

				float NextFloat( float min , float max );

				/// Gives a random unsigned integer. may be any value.

				unsigned NextUnsigned();
				
				/// Gives a random unsigned integer between the two values.
				
				unsigned NextUnsigned( unsigned min , unsigned max );
				
				/// Gives a random signed integer. may be any value.
				
				int NextInt();
				
				/// Gives a signed integer between the two given values
				
				int NextInt( int min , int max );
				
				/// Gives a random boolean value
				
				bool NextBool(){ return (((unsigned)(NextInt())) & 1 ) == 1; }
				
				/// Randomises the random number generator. Picking a random seed;
				
				void Randomize();
	
		private:
			
			//=========================================================================================================
			// Variables
			//=========================================================================================================
			
				/// The current seed for the random number generator: determines the next random number
				
				unsigned m_seed;
	};
};

#endif 
