/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef BULLET_MANAGER_H
#define BULLET_MANAGER_H

//=====================================================================================================================
//=====================================================================================================================

#include "GameObjects\MoveableObject.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	//=================================================================================================================
	
	class Camera2D;
	class BulletBase;
	
	//=================================================================================================================
	/// @brief		This class manages a set of bullets. It draws each of the bullets, and allows bullets to fire- 
	///				as well as managing their movement.
	/// @remarks	Unlike other managers (AI managers and such) this class is not a game object. The reason for this
	///				is because it is designed to be used by either a game object (such as player ship) or by an
	///				object manager that is placed in the game (such as EnemyWaspManager)
	//=================================================================================================================

	class BulletManager : public ObjectBase
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================
			
				/// Creates the bullet manager
				
				BulletManager( unsigned bullet_count , Stack & stack , Color bullet_color , float bullet_length );
				
				/// Destroys the bullet manager
				
				virtual ~BulletManager();				
				
				/// Updates all bullets in the manager
				
				virtual void UpdateAll( float time_delta );
				
				/// Draws all bullets in the manager
				
				void DrawAll( Camera2D * camera );
				
				/// Fires one bullet: if there are no free bullets then the oldest allocated bullet is scrapped and reused
				
				void FireBullet( const Vec2f & pos , float angle , float base_speed );

				/// Returns the color used for bullets in this manager

				const Color & GetBulletColor(){ return m_bullet_color; }
				
		private:
		
			//=========================================================================================================
			// Variables
			//=========================================================================================================
			
				/// Color to draw all the bullets in
				
				Color m_bullet_color;
				
				/// How long all the bullets should be when drawing
				
				float m_bullet_length;
				
				/// Number of bullets the manager has
				
				unsigned m_bullet_count;
				
				/// A pool containing all of the bullets in use
				
				Pool<BulletBase> * m_bullet_pool;
				
				/// An array containing pointers to all of the bullets we allocated
				
				BulletBase ** m_bullets;
	};
};

#endif
