/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef ENEMY_BULLET_MANAGER_H
#define ENEMY_BULLET_MANAGER_H

//=====================================================================================================================
//=====================================================================================================================

#include "Game\BulletManager.h"
#include "Graphics\Texture.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	//=================================================================================================================
	
	class PlayerShip;
	class AIManager;
	
	//=================================================================================================================
	/// @brief		This is a simple extension of the ordinary bullet manager that caches a reference to the player ship
	///				so that enemy bullets can quickly check for collisions with it.
	//=================================================================================================================

	class EnemyBulletManager : public BulletManager
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================

				/// Makes the bullet manager
			
				EnemyBulletManager( AIManager * ai_manager , unsigned bullet_count , Stack & stack , Color bullet_color , float bullet_length );
				
				/// Destructor: 
				
				virtual ~EnemyBulletManager();
				
				/// Tells the object to announce it's type name: this is the type name without the Game:: namespace

				virtual const char * GetTypeName() const;				
				
				/// Update all function: updates reference to the player object as well as doing base class logic
				
				virtual void UpdateAll( float time_delta );
				
				/// Returns the player's ship
				
				inline PlayerShip * GetPlayerShip(){ return m_player; }

				/// Returns the texture used for bullets particles and explosions
				
				inline Texture & GetParticleTexture(){ return m_particle_texture; }
				
				/// Gets the ai manager this bullet manager belongs to
				
				inline AIManager * GetAIManager(){ return m_ai_manager; }
				
		private:
		
			//=========================================================================================================
			// Variables
			//=========================================================================================================
			
				/// A cached reference to the player's ship, if there
				
				PlayerShip * m_player;

				/// Texture to use for the bullet particles
				
				Texture m_particle_texture;
				
				/// The AI manager this bullet manager belongs to
				
				AIManager * m_ai_manager;
	};
};

#endif
