/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//=====================================================================================================================
/// @brief		This simple class populates the level.
//=====================================================================================================================

#include "Precompiled.h"
#include "Game\Level.h"
#include "Game\GameObjectManager.h"
#include "GameObjects\PlayerShip.h"
#include "GameObjects\Background.h"
#include "GameObjects\EnemyFactoryManager.h"
#include "GameObjects\EnemyWaspManager.h"
#include "GameObjects\EnemyPredatorManager.h"
#include "GameObjects\ObstacleManager.h"

using namespace Game;

//=====================================================================================================================
/// @brief		Populates the level
/// @param		m - game object manager which holds all of the level objects
/// @pre		Given manager must not be null
//=====================================================================================================================

void Level::PopulateLevel( GameObjectManager * m )
{
	// Make sure ok:

	ENSURE( m , "Given manager must not be null" );

	// Make a player:

	{
		// Make: 

		PlayerShip * p = new PlayerShip();

		// Add to the level:

		m->AddObject(p);

		// Done with this object

		p->RemoveReference();
	}
	
	// Background:

	{
		// Make: 

		Background * o = new Background("Textures\\Background.dds");

		// Add to the level:

		m->AddObject(o);

		// Done with this object

		o->RemoveReference();
	}
	
	// For making factories
	
	{
		// Make: 

		EnemyFactoryManager * o = new EnemyFactoryManager
		(
			256,		// Maximum enemies
			128,		// Initial enemies
			15.25f,		// Min time between spawns
			50.5f		// Max time between spawns
		);

		// Add to the level:

		m->AddObject(o);

		// Done with this object

		o->RemoveReference();
	}
	
	// For making wispas
	
	{
		// Make: 

		EnemyWaspManager * o = new EnemyWaspManager
		(
			512,		// Maximum enemies
			0,			// Initial enemies
			35.25f,		// Min time between spawns
			120.5f		// Max time between spawns
		);

		// Add to the level:

		m->AddObject(o);

		// Done with this object

		o->RemoveReference();
	}
	
	// For making predators
	
	{
		// Make: 

		EnemyPredatorManager * o = new EnemyPredatorManager
		(
			256,		// Maximum enemies
			0,			// Initial enemies
			55.25f,		// Min time between spawns
			200.5f		// Max time between spawns
		);

		// Add to the level:

		m->AddObject(o);

		// Done with this object

		o->RemoveReference();
	}
	
	// For making obstacles n stuff
	
	{
		// Make: 

		ObstacleManager * o = new ObstacleManager(16);

		// Add to the level:

		m->AddObject(o);

		// Done with this object

		o->RemoveReference();
	}
	
	// Send the level ready message:
	
	m->SendGlobalEvent( GameObjectManager::GOM_LEVEL_READY , 0 );
}