/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef PLAYER_BULLET_MANAGER_H
#define PLAYER_BULLET_MANAGER_H

//=====================================================================================================================
//=====================================================================================================================

#include "Game\BulletManager.h"
#include "Graphics\Texture.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	//=================================================================================================================
	
	class EnemyWaspManager;
	class EnemyFactoryManager;
	class EnemyPredatorManager;
	class PlayerShip;
	
	//=================================================================================================================
	/// @brief		This is a simple extension of the ordinary bullet manager that caches references to AI object
	///				managers for the purposes of collision detection.
	//=================================================================================================================

	class PlayerBulletManager : public BulletManager
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================
			
				/// Creates the player bullet manager
				
				PlayerBulletManager( PlayerShip * ship , unsigned bullet_count , Stack & stack , Color bullet_color , float bullet_length );
				
				/// Destructor: 
				
				virtual ~PlayerBulletManager();
				
				/// Tells the object to announce it's type name: this is the type name without the Game:: namespace

				virtual const char * GetTypeName() const;				
				
				/// Update all function: updates references to AI object managers as well as doing base class logic
				
				virtual void UpdateAll( float time_delta );
				
				/// Returns the enemy wasp manager
				
				inline EnemyWaspManager * GetEnemyWaspManager(){ return m_wasp_mgr; }
				
				/// Returns the enemy factory manager
				
				inline EnemyFactoryManager * GetEnemyFactoryManager(){ return m_factory_mgr; }
				
				/// Returns the enemy predator manager
				
				inline EnemyPredatorManager * GetEnemyPredatorManager(){ return m_predator_mgr; }
				
				/// Returns the texture used for bullets particles and explosions
				
				inline Texture & GetParticleTexture(){ return m_particle_texture; }
				
				/// Gets the player ship this belongs to
				
				inline PlayerShip * GetPlayerShip(){ return m_player_ship; }
				
		private:
		
			//=========================================================================================================
			// Variables
			//=========================================================================================================
			
				/// Pointer to the wasp manager
				
				EnemyWaspManager * m_wasp_mgr;
				
				/// Pointer to the factory manager
				
				EnemyFactoryManager * m_factory_mgr;				
				
				/// Pointer to the predator manager
				
				EnemyPredatorManager * m_predator_mgr;
				
				/// Texture to use for the bullet particles
				
				Texture m_particle_texture;	
				
				/// Player ship this manager belongs to
				
				PlayerShip * m_player_ship;	
	};
};

#endif
