/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef BACKGROUND_H
#define BACKGROUND_H

//=====================================================================================================================
//=====================================================================================================================

#include "GameObjects\GameObjectBase.h"
#include "Graphics\Texture.h"
#include "Graphics\VertexShader.h"
#include "Graphics\PixelShader.h"
#include "Graphics\ShaderUniform.h"
#include "Graphics\VertexFormat.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	//=================================================================================================================
	
	class Camera2D;

	//=================================================================================================================
	/// @brief		This class displays a textured background that scrolls as the camera moves.
	//=================================================================================================================

	class Background : public GameObjectBase
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================
			
				/// Creates the background
				
				Background( const std::string & texture_name );
				
				/// Draw function: draws the background
				
				virtual void OnDraw( Camera2D * c );
				
				/// Returns the type name for this object

				virtual const char * GetTypeName() const;
				
		private:
		
			//=========================================================================================================
			// Constants
			//=========================================================================================================
			
				/// Area of the game world over which the background area is shown
				
				static const Vec2f BACKGROUND_TEX_AREA;
		
			//=========================================================================================================
			// Structures
			//=========================================================================================================
			
				/// Vertex format used by the background
			
				struct Vertex
				{
					/// Position of the vertex
				
					Vec4f m_pos;
				
					/// Color of the vertex
					
					Vec4f m_col;
					
					/// Texture coordinate of the vertex
					
					Vec2f m_tex;
				};
		
			//=========================================================================================================
			// Variables
			//=========================================================================================================
			
				/// Texture used by the background
				
				Texture m_texture;
				
				/// Vertex shader used
				
				VertexShader m_vertex_shader;
				
				/// Pixel shader used	
				
				PixelShader m_pixel_shader;
				
				/// Transform uniform on the vertex shader
				
				ShaderUniform m_u_transform;
				
				/// Texture uniform used by pixel shader
				
				ShaderUniform m_u_texture;
				
				/// Vertex format used by the background
				
				VertexFormat m_vertex_format;
				
				/// Vertices used to draw the background
				
				Vertex m_vertices[4];
				
				/// True if the background has valid graphics resources:
				
				bool m_valid_graphics_resources;
	};
};

#endif
