/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef BUFFERED_POINT_RENDERER_H
#define BUFFERED_POINT_RENDERER_H

//=====================================================================================================================
//=====================================================================================================================

#include "Graphics\VertexShader.h"
#include "Graphics\PixelShader.h"
#include "Graphics\VertexFormat.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	//=================================================================================================================
	
	class Window;
	class Display;
	class PixelShaderManager;
	class VertexShaderManager;

	//=================================================================================================================
	/// @brief		Class representing a buffered point renderer. The buffered point renderer allows for points to be 
	///				rendered in a batch, rather than one by one as with the primitives renderer.
	//=================================================================================================================
	
	class BufferedPointRenderer : public ObjectBase
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================

				/// Creates the buffered point renderer

				BufferedPointRenderer( Display * display , VertexShaderManager * vsmgr , PixelShaderManager * psmgr );

				/// Destroys the buffered point renderer

				virtual ~BufferedPointRenderer();

				/// Draws a set of points immediately, that are specified in the given stack. See details for format of data on the stack

				void Draw( const Matrix4f & transform , unsigned count , Stack & stack );

				/// Tells if this renderer is valid and setup correctly

				inline bool IsValid(){ return m_is_valid; }

				/// Returns the name of this type

				virtual const char * GetTypeName() const;

		private:

			//=========================================================================================================
			// Functions
			//=========================================================================================================

				/// Sets up the state for the buffered point renderer

				void SetupRenderState( const Matrix4f & transform );

			//=========================================================================================================
			// Variables
			//=========================================================================================================
				
				/// Pixel shader used by the buffered point renderer

				PixelShader m_ps;

				/// Vertex shader used by the buffered point renderer

				VertexShader m_vs;

				/// Vertex format used by the buffered point renderer

				VertexFormat m_vf;

				/// Shader uniform for the transform variable on the vertex shader

				ShaderUniform m_u_transform;

				/// True if this renderer is valid

				bool m_is_valid;

				/// Pointer to the direct3d device that the renderer uses

				IDirect3DDevice9 * m_device;
	};

};

#endif 