/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef SHADER_UNIFORM_H
#define SHADER_UNIFORM_H

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	//=================================================================================================================
	
	class Display;
	class VertexShader;
	class PixelShader;
	class Texture;
	
	//=================================================================================================================
	/// @brief		Represents a uniform variable in a vertex or pixel shader.
	//=================================================================================================================

	class ShaderUniform
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================

				/// Default constructor: creates an invalid uniform

				ShaderUniform();

				/// Creates the shader uniform object: call shaders to create rather than use this directly

				ShaderUniform( IDirect3DDevice9 * m_d3d_device , ID3DXConstantTable * constant_table , D3DXHANDLE constant_table_handle );

				/// Destroys the shader uniform object

				~ShaderUniform();

				/// Copy constructor:

				ShaderUniform( const ShaderUniform & other );

				/// Assignment operator:

				ShaderUniform & operator = ( const ShaderUniform & other );

				//=========================================================================================================
				/// @brief		Tells if this variable is valid 
				/// @returns	True if the shader uniform is valid
				//=========================================================================================================

				inline bool IsValid() const { return m_constant_handle != 0 && m_d3d_device != 0 && m_constant_table != 0; }

				/// Sets the constant as a float

				void SetFloat( float value ) const;
				
				/// Sets the constant as a vector 2:
				
				void SetVec2f( const Vec2f & value ) const;
				
				/// Sets the constant as a vector 3:
				
				void SetVec3f( const Vec3f & value ) const;
				
				/// Sets the constant as a vector 4:
				
				void SetVec4f( const Vec4f & value ) const;

				/// Sets the constant as a boolean

				void SetBool( bool value ) const;

				/// Sets the constant as a integer 

				void SetInt( int value ) const;

				/// Sets the constant as a 4x4 matrix

				void SetMatrix( const Matrix4f & matrix ) const;
				
				/// If this is a texture uniform this function returns which sampler unit the texture should be bound to
				
				int GetSamplerUnit() const;
				
				/// Binds the given texture to the texture unit of the shader uniform
				
				void BindTexture( Texture & t );
				
		private:			
		
			//=========================================================================================================
			// Variables
			//=========================================================================================================

				/// Handle to a variable from the Direc3D constant table

				ID3DXConstantTable * m_constant_table;

				/// Handle to a variable from the Direc3D constant table

				D3DXHANDLE m_constant_handle;

				/// Direc3D device the uniform belongs to

				IDirect3DDevice9 * m_d3d_device;
	};

};

#endif 