/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//=====================================================================================================================
/// @brief		Represents a loaded texture resource. Internally this class is just storing a managed resource, 
///				but this is the end user interface for textures so the class comes with additional functionality.
//=====================================================================================================================

#include "Precompiled.h"
#include "Graphics\Texture.h"

using namespace Game;

//=====================================================================================================================
///	@brief	Applys the texture to the given sampler unit
///	@param	sampler_unit - texture unit to apply the texture to
//=====================================================================================================================

void Texture::Apply( unsigned sampler_unit ) const
{
	// Get the actual texture
	
	IDirect3DTexture9 * texture = m_resource.GetResource()->GetData();

	// See if we have an actual texture:
	
	if ( texture )
	{
		// Ok - good. Now get the Direct3D device:
		
		IDirect3DDevice9 * device = (IDirect3DDevice9*) m_resource.GetManagerUserData();
		
		// See if there:
		
		if ( device )
		{
			// Apply the texture:
			
			device->SetTexture( sampler_unit , texture );
		}
	}
}	

//=====================================================================================================================
///	@brief		Gets the width of the texture
///	@returns	Width of the texture
//=====================================================================================================================

int Texture::GetWidth() const
{
	// Get the actual texture
	
	IDirect3DTexture9 * texture = m_resource.GetResource()->GetData();

	// See if there:
	
	if ( texture )
	{
		// Store the surface description here:
		
		D3DSURFACE_DESC surface_desc;
		
		// Get the surface description for the texture:
		
		HRESULT result = texture->GetLevelDesc(0,&surface_desc);
		
		// See if that was ok:
		
		if ( result == D3D_OK )
		{
			// Return the width
			
			return surface_desc.Width;
		}
		else
		{
			// Failed: return 0
			
			return 0;
		}	
	}
	else
	{
		// Not there: return 0
		
		return 0;
	}
}

//=====================================================================================================================
///	@brief		Gets the height of the texture
///	@returns	Height of the texture
//=====================================================================================================================

int Texture::GetHeight() const
{
	// Get the actual texture
	
	IDirect3DTexture9 * texture = m_resource.GetResource()->GetData();

	// See if there:
	
	if ( texture )
	{
		// Store the surface description here:
		
		D3DSURFACE_DESC surface_desc;
		
		// Get the surface description for the texture:
		
		HRESULT result = texture->GetLevelDesc(0,&surface_desc);
		
		// See if that was ok:
		
		if ( result == D3D_OK )
		{
			// Return the height
			
			return surface_desc.Height;
		}
		else
		{
			// Failed: return 0
			
			return 0;
		}	
	}
	else
	{
		// Not there: return 0
		
		return 0;
	}
}
