/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef VERTEX_SHADER_H
#define VERTEX_SHADER_H

//=====================================================================================================================
//=====================================================================================================================

#include "Resources\VertexShaderManager.h"
#include "Graphics\ShaderUniform.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	/// @brief		Represents a vertex shader. Vertex and pixel shaders must be used in conjunction to draw. 
	//=================================================================================================================
	
	class VertexShader
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================
			
				//=====================================================================================================
				/// @brief	Creates a default invalid vertex shader.
				//=====================================================================================================
				
				VertexShader(){}

				//=====================================================================================================
				/// @brief	Creates the vertex shader from the given managed resource containing a vertex shader
				/// @param	resource - managed resource used by the vertex shader
				//=====================================================================================================
				
				VertexShader( VertexShaderManager::ManagedResource resource ) : m_resource(resource){}
				
				//=====================================================================================================
				///	@brief		Returns the managed resource used by the pixel shader
				///	@returns	Managed resource used by the manager
				//=====================================================================================================
				
				VertexShaderManager::ManagedResource GetResource() const { return m_resource; }			

				//=====================================================================================================
				/// @brief Puts the vertex shader into use and binds it to the pipeline
				//=====================================================================================================
				
				void Apply() const;
				
				//=====================================================================================================
				/// @brief		Gets the given shader uniform
				///	@param		name - name of the shader uniform
				/// @returns	The given shader uniform			
				//=====================================================================================================
				
				ShaderUniform GetUniform( const char * name ) const;
				
				//=====================================================================================================
				/// @brief		Gets the given shader uniform
				///	@param		name - name of the shader uniform
				/// @returns	The given shader uniform
				//=====================================================================================================
				
				ShaderUniform GetUniform( const std::string & name ) const;			

				//=====================================================================================================
				///	@brief		Tells if the resource is valid
				///	@returns	True if the resource is valid
				//=====================================================================================================

				inline bool IsValid() const { return m_resource.IsValid(); }
				
				//=====================================================================================================
				///	@brief		Assignment operator
				/// @param		other - the other vertex shader resource
				/// @returns	this resource
				//=====================================================================================================			
				
				inline VertexShader & operator = ( const VertexShader & other )
				{
					// Only if not the same:
					
					if ( &other != this )
					{
						// Do the assignment 
						
						m_resource = other.m_resource;
					}
					
					// Return this resource
					
					return * this;		
				}
				
				//=====================================================================================================
				///	@brief		Copy constructor
				/// @param		other - the other vertex shader resource
				//=====================================================================================================			
				
				VertexShader( const VertexShader & other ) : m_resource( other.m_resource ){}			

		private:

			//=========================================================================================================
			// Variables
			//=========================================================================================================
			
				/// The actual resource manager resource used by the vertex shader
				
				VertexShaderManager::ManagedResource m_resource;
	};
	
};

#endif 