/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//=====================================================================================================================
/// @brief		Allows for gamepad input using the XINPUT library
//=====================================================================================================================

#include "Precompiled.h"
#include "Input\Gamepad.h"
#include <cstring>

using namespace Game;

//=====================================================================================================================
/// Statics
//=====================================================================================================================

/// Last known state of gamepad 1

Gamepad::GamepadState Gamepad::s_state1;

/// Last known state of gamepad 2
		
Gamepad::GamepadState Gamepad::s_state2;

/// Last known state of gamepad 3

Gamepad::GamepadState Gamepad::s_state3;

/// Last known state of gamepad 4

Gamepad::GamepadState Gamepad::s_state4;

//=====================================================================================================================
/// @brief	Initialises the gamepad interface:
//=====================================================================================================================

void Gamepad::Initialize()
{
	// Clear all of the input structures:

	memset( &s_state1 , 0 , sizeof(GamepadState) );
	memset( &s_state2 , 0 , sizeof(GamepadState) );
	memset( &s_state3 , 0 , sizeof(GamepadState) );
	memset( &s_state4 , 0 , sizeof(GamepadState) );

	s_state1.m_ltrigger = 0;
	s_state2.m_ltrigger = 0;
	s_state3.m_ltrigger = 0;
	s_state4.m_ltrigger = 0;

	s_state1.m_rtrigger = 0;
	s_state2.m_rtrigger = 0;
	s_state3.m_rtrigger = 0;
	s_state4.m_rtrigger = 0;

	s_state1.m_lstick_x = 0;
	s_state2.m_lstick_x = 0;
	s_state3.m_lstick_x = 0;
	s_state4.m_lstick_x = 0;

	s_state1.m_lstick_y = 0;
	s_state2.m_lstick_y = 0;
	s_state3.m_lstick_y = 0;
	s_state4.m_lstick_y = 0;

	s_state1.m_rstick_x = 0;
	s_state2.m_rstick_x = 0;
	s_state3.m_rstick_x = 0;
	s_state4.m_rstick_x = 0;

	s_state1.m_rstick_y = 0;
	s_state2.m_rstick_y = 0;
	s_state3.m_rstick_y = 0;
	s_state4.m_rstick_y = 0;
}

//=====================================================================================================================
/// @brief	Gets the state of the given controller if possible
/// @param	player_index - index of the player to get the controls for
/// @param	use_dead_zones - whether or not dead zones will be applied to zero the analog input
//=====================================================================================================================

Gamepad::GamepadState Gamepad::GetState( Gamepad::PlayerIndex player_index , bool use_dead_zones )
{
	// Store the state here:

	XINPUT_STATE state; 

	// Try and get it:

	HRESULT result = XInputGetState( (DWORD) player_index , & state );

	// Get a pointer to the relevant stored state structure:

	GamepadState * s_state = 0;

	switch ( player_index )
	{
		case PLAYER_ONE:	s_state = & s_state1;	break;
		case PLAYER_TWO:	s_state = & s_state2;	break;
		case PLAYER_THREE:	s_state = & s_state3;	break;
		default:			s_state = & s_state4;	break;
	}

	// See if that failed:

	if ( result == ERROR_SUCCESS )
	{
		// Update the state of the global struct:

		s_state->m_connected	= true;

		s_state->m_a			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_A				) != 0;
		s_state->m_b			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_B				) != 0;
		s_state->m_x			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_X				) != 0;
		s_state->m_y			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_Y				) != 0;
		s_state->m_up			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_DPAD_UP		) != 0;
		s_state->m_down			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_DPAD_DOWN		) != 0;
		s_state->m_left			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_DPAD_LEFT		) != 0;
		s_state->m_right		= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_DPAD_RIGHT		) != 0;
		s_state->m_lstick		= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_LEFT_THUMB		) != 0;
		s_state->m_rstick		= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_RIGHT_THUMB	) != 0;
		s_state->m_lshoulder	= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_LEFT_SHOULDER	) != 0;
		s_state->m_rshoulder	= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_RIGHT_SHOULDER	) != 0;
		s_state->m_start		= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_START			) != 0;
		s_state->m_back			= ( state.Gamepad.wButtons &  XINPUT_GAMEPAD_BACK			) != 0;

		// LT

		if ( use_dead_zones && state.Gamepad.bLeftTrigger < XINPUT_GAMEPAD_TRIGGER_THRESHOLD )
		{
			// Within dead zone:

			s_state->m_ltrigger = 0;
		}
		else
		{
			// Ok: convert to float as normal

			s_state->m_ltrigger = (float)(state.Gamepad.bLeftTrigger) / 255.0f;		
		}

		// RT

		if ( use_dead_zones && state.Gamepad.bRightTrigger < XINPUT_GAMEPAD_TRIGGER_THRESHOLD )
		{
			// Within dead zone:

			s_state->m_rtrigger = 0;
		}
		else
		{
			// Ok: convert to float as normal

			s_state->m_rtrigger = (float)(state.Gamepad.bRightTrigger) / 255.0f;		
		}

		// LSX

		if 
		( 
			use_dead_zones 
			&& 
			state.Gamepad.sThumbLX <   XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE && 
			state.Gamepad.sThumbLX > - XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE
		)
		{
			// Within dead zone:

			s_state->m_lstick_x = 0;
		}
		else
		{
			// Now to float and bring to -1 to 1 range

			if ( state.Gamepad.sThumbLX < 0 )
			{
				s_state->m_lstick_x = ((float)(state.Gamepad.sThumbLX)) / 32768.0f;
			}
			else
			{
				s_state->m_lstick_x = ((float)(state.Gamepad.sThumbLX)) / 32767.0f;
			}
		}

		// LSY

		if 
		( 
			use_dead_zones 
			&& 
			state.Gamepad.sThumbLY <   XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE && 
			state.Gamepad.sThumbLY > - XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE
		)
		{
			// Within dead zone:

			s_state->m_lstick_y = 0;
		}
		else
		{
			// Now to float and bring to -1 to 1 range

			if ( state.Gamepad.sThumbLY < 0 )
			{
				s_state->m_lstick_y = ((float)(state.Gamepad.sThumbLY)) / 32768.0f;
			}
			else
			{
				s_state->m_lstick_y = ((float)(state.Gamepad.sThumbLY)) / 32767.0f;
			}
		}

		// RSX

		if 
		( 
			use_dead_zones 
			&& 
			state.Gamepad.sThumbRX <   XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE && 
			state.Gamepad.sThumbRX > - XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE
		)
		{
			// Within dead zone:

			s_state->m_rstick_x = 0;
		}
		else
		{
			// Now to float and bring to -1 to 1 range

			if ( state.Gamepad.sThumbRX < 0 )
			{
				s_state->m_rstick_x = ((float)(state.Gamepad.sThumbRX)) / 32768.0f;
			}
			else
			{
				s_state->m_rstick_x = ((float)(state.Gamepad.sThumbRX)) / 32767.0f;
			}
		}

		// RSY

		if 
		( 
			use_dead_zones 
			&& 
			state.Gamepad.sThumbRY <   XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE && 
			state.Gamepad.sThumbRY > - XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE
		)
		{
			// Within dead zone:

			s_state->m_rstick_y = 0;
		}
		else
		{
			// Now to float and bring to -1 to 1 range

			if ( s_state->m_rstick_y < state.Gamepad.sThumbRY )
			{
				s_state->m_rstick_y = ((float)(state.Gamepad.sThumbRY)) / 32768.0f;
			}
			else
			{
				s_state->m_rstick_y = ((float)(state.Gamepad.sThumbRY)) / 32767.0f;
			}
		}
	}
	else
	{
		// Not connected: just use current state and set connected to false

		s_state->m_connected = false;
	}

	// Return the current state 

	return * s_state;
}