/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef BITMAP_FONT_RESOURCE_DATA_H
#define BITMAP_FONT_RESOURCE_DATA_H

//=====================================================================================================================
//=====================================================================================================================

#include "Graphics\Texture.h"
#include "Graphics\BitmapFontSettings.h"

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
    //=================================================================================================================
    /// @brief		This structure holds all of the data for a bitmap font resource. Since resources can only point 
    ///				to one thing (bar the extra user field which allows another thing to be pointed to) it is 
    ///				necessary to aggregate all of the data for a font into one object. This is the purpose of this 
    ///				structure.
	//=================================================================================================================

    struct BitmapFontResourceData
    {
		public:
		
			//=========================================================================================================
			// Structures
			//=========================================================================================================
			
				//=====================================================================================================
				/// @brief		This structure holds sizing and positional information for one character in the font 
				///				texture. It basically holds the u and v coordinate position of the character as well
				///				as the u and v size of the character.
				//=====================================================================================================

				struct CharacterMetric
				{
					/// Position of the character in the texture (X). 0-1 texture coordinate. </summary>

					float m_u;

					/// Position of the character in the texture (Y). 0-1 texture coordinate. </summary>

					float m_v;

					/// Width of the character in the texture. 0-1 texture coordinate. </summary>

					float m_s;

					/// Height of the character in the texture. 0-1 texture coordinate. </summary>

					float m_t;
				};
    
			//=========================================================================================================
			// Variables
			//=========================================================================================================
	    
				/// Metrics for each character in the font
				
				CharacterMetric m_metrics[256];
				
				/// Settings for the font
				
				BitmapFontSettings m_settings;
				
				/// Texture used by the font
				
				Texture m_texture;
				
			//=========================================================================================================
			// Functions
			//=========================================================================================================
			
				//=====================================================================================================
				/// @brief		Default constructor: initialises the bitmap font data
				//=====================================================================================================
				
				BitmapFontResourceData()
				{
					// Everything else is initialised: just need to init the character metrics
					
					for ( int i = 0 ; i < 256 ; ++i )
					{
						m_metrics[i].m_u = 0;
						m_metrics[i].m_v = 0;
						m_metrics[i].m_s = 0;
						m_metrics[i].m_t = 0;						
					}
				}
				
	};
}
    
#endif