/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//=====================================================================================================================
/// @brief		Type of resource manager that looks after loading pixel shaders.
//=====================================================================================================================

#include "Precompiled.h"
#include "Resources\PixelShaderManager.h"
#include "Graphics\PixelShader.h"
#include "Graphics\Display.h"

using namespace Game;

//=====================================================================================================================
/// @brief		Constructor. Creates the pixel shader manager.
/// @param		display - a direct3d display the pixel shader manager is to use
/// @pre		There must be a display given
/// @pre		The display must have a valid device
//=====================================================================================================================

PixelShaderManager::PixelShaderManager( Display * display ) : ResourceManager( (long) display->GetDevice() )
{
	// Make sure ok:
	
	ENSURE( display					, "There must be a direct3d device given"	)
	ENSURE( display->GetDevice()	, "The display must have a valid device"	)

	// Add a reference to device given:
	
	display->GetDevice()->AddRef();
}

//=====================================================================================================================
/// @brief		Destructor. Destroys the pixel shader manager.
//=====================================================================================================================

PixelShaderManager::~PixelShaderManager()
{
	// Get the graphics device:
	
	IDirect3DDevice9 * device = (IDirect3DDevice9*) m_user_data;
	
	// Remove the reference to our device:
	
	device->Release();
}

//=====================================================================================================================
/// @brief		Gets the given pixel shader: this shader must be in binary precompiled format
/// @param		name - name of the resource to get
/// @returns	A new pixel shader object, which is a high level representation of the managed resource
//=====================================================================================================================

PixelShader PixelShaderManager::GetPixelShader( const std::string & name )
{
	// Get the vertex shader resource
	
	PixelShaderManager::ManagedResource resource = GetResource(name);
	
	// Return a new vertex shader
	
	return PixelShader( resource );
}

//=====================================================================================================================
/// @brief		Gets the given pixel shader: this shader must be in binary precompiled format
/// @param		name - name of the resource to get
/// @returns	A new pixel shader object, which is a high level representation of the managed resource
//=====================================================================================================================

PixelShader PixelShaderManager::GetPixelShader( const char * name )
{
	// Get the vertex shader resource
	
	PixelShaderManager::ManagedResource resource = GetResource(name);
	
	// Return a new vertex shader
	
	return PixelShader( resource );
}

//=====================================================================================================================
/// @brief		Annouces the type name for this object.
/// @returns	Type name of this object
//=====================================================================================================================

const char * PixelShaderManager::GetTypeName()
{
	return "PixelShaderManager";
}
