/*
	Trig Wars.
	Copyright (c) 2008, Darragh Coy.
	All rights reserved.
	Original work(s), source code & author(s) details available at: http://www.darraghcoy.com

	Redistribution and use in source and binary forms, with or without modification, are permitted 
	provided that the following conditions are met:

		* 	Redistributions of source code must retain the above copyright notice, this list of 
			conditions and the following disclaimer.
		* 	Redistributions in binary form must reproduce the above copyright notice, this list of 
			conditions and the following disclaimer in the documentation.
		* 	Neither the name of the author nor the names of contributors may be used to endorse or 
			promote products derived from this software without specific prior written permission.
		* 	The use of any third party libraries, modules or data files provided with the source 
			form of the program must follow and respect the individual licensing terms of these 
			components. You do not have permission to modify or circumvent the licensing terms for 
			provided third party components unless the third party licenses specifically allow this. 
			If you do not agree to the terms of provided third party components then you do not have 
			permission to use them in your derived work(s).
		* 	The terms of this license do not apply to separate (but related) distributions if supplied 
			by the author(s), only to the contents of this source distribution, derived works, and the 
			binaries produced by either modified or unmodified versions of the source code. The terms 
			of this license do not also apply to any third party libraries or components provided with 
			the source distribution. Such third party elements are licensed under their own separate 
			agreements.

	THIS SOFTWARE IS PROVIDED BY DARRAGH COY ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL DARRAGH COY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef WINDOW_EVENT_LISTENER_H
#define WINDOW_EVENT_LISTENER_H

//=====================================================================================================================
//=====================================================================================================================

namespace Game
{
	//=================================================================================================================
	/// @brief		Serves as base class for derived event listeners. The behaviour
	///				of the Window can be changed on events by implemented a derived
	///				version of this class and setting it as the window event listener.
	//=================================================================================================================

	class WindowEventListener : public ObjectBase
	{
		public:

			//=========================================================================================================
			// Functions
			//=========================================================================================================

			// No unsed param warnings
		
			#pragma warning( disable : 4100 )

				//=====================================================================================================
				///	@brief		Called when the mouse is moved over the window
				/// @param		x - x position of the mouse in client coordinates
				/// @param		y - y position of the mouse in client coordinates
				//=====================================================================================================

				virtual void OnMouseMoved( int x , int y ){;}

				//=====================================================================================================
				///	@brief		Called when the mouse pressed over the window
				/// @param		x - x position of the mouse in client coordinates
				/// @param		y - y position of the mouse in client coordinates
				///	@param		button_vk - virtual key code of the button
				//=====================================================================================================

				virtual void OnMousePressed( int x , int y , int button_vk ){;}

				//=====================================================================================================
				///	@brief		Called when the mouse released over the window
				/// @param		x - x position of the mouse in client coordinates
				/// @param		y - y position of the mouse in client coordinates
				///	@param		button_vk - virtual key code of the button
				//=====================================================================================================

				virtual void OnMouseReleased( int x , int y , int button_vk	){;}

				//=====================================================================================================
				///	@brief		Called when the mouse wheel is scrolled over the window
				///	@param		scroll_amount - amount of scroll from the wheel. Units are either positive or negative
				///				and one whole unit represents a 'click' of the wheel.
				//=====================================================================================================

				virtual void OnMouseWheelScrolled( float scroll_amount ){;}

				//=====================================================================================================
				///	@brief		Called when the keyboard is pressed
				/// @param		key_code - Virtual key code of the key pressed
				//=====================================================================================================

				virtual void OnKeyboardPressed( Keyboard::Key key_code ){;}

				//=====================================================================================================
				///	@brief		Called when the keyboard is pressed
				/// @param		key_code - Virtual key code of the key pressed
				//=====================================================================================================

				virtual void OnKeyboardReleased( Keyboard::Key key_code ){;}

				//=====================================================================================================
				///	@brief		Called when the window is resized
				//=====================================================================================================

				virtual void OnWindowResized(){;}

				//=====================================================================================================
				///	@brief		Called when the user has asked to close th window
				//=====================================================================================================

				virtual void OnWindowClosing(){;}

				//=====================================================================================================
				///	@brief		Called after the window has been closed
				//=====================================================================================================

				virtual void OnWindowClosed(){;}

				//=====================================================================================================
				///	@brief		Called when the window is redrawing itself
				//=====================================================================================================

				virtual void OnWindowRedrawing(){;}

				//=====================================================================================================
				///	@brief		Called when the has lost focus
				//=====================================================================================================

				virtual void OnWindowFocusLost(){;}

				//=====================================================================================================
				///	@brief		Called when the has gained focus
				//=====================================================================================================

				virtual void OnWindowFocusGained(){;}

			// Renable unsed param warnings
		
			#pragma warning( default : 4100 )
	};
	
};

#endif
